@@ -80,7 +80,7 @@ setMethod("dbDisconnect", "OdbcConnection",
 #'   SQLPrepare, and the `params` argument is ignored
 #' @export
 setMethod("dbSendQuery", c("OdbcConnection", "character"),
-  function(conn, statement, params = NULL, ..., immediate = is.null(params)) {
+  function(conn, statement, params = NULL, ..., immediate = FALSE) {
     OdbcResult(
       connection = conn,
       statement = statement,
@@ -90,6 +90,18 @@ setMethod("dbSendQuery", c("OdbcConnection", "character"),
   }
 )
 
+#' @rdname OdbcConnection
+#' @inheritParams DBI::dbExecute
+#' @export
+setMethod("dbExecute", c("OdbcConnection", "character"),
+  function(conn, statement, params = NULL, ..., immediate = is.null(params)) {
+    rs <- dbSendStatement(conn, statement, params = params, ..., immediate = immediate)
+    on.exit(dbClearResult(rs))
+
+    dbGetRowsAffected(rs)
+  }
+)
+
 #' @rdname OdbcConnection
 #' @inheritParams DBI::dbSendStatement
 #' @param params Query parameters to pass to [dbBind()].
@@ -202,7 +214,7 @@ setMethod("dbListFields", c("OdbcConnection", "character"),
            schema_name = NULL,
            column_name = NULL,
            ...) {
-    cols <- odbcConnectionColumns(
+    cols <- odbcConnectionColumns_(
       conn,
       name = name,
       catalog_name = catalog_name,
